package com.recordingblogs.oreffect;

/**
 This interface should be implemented by any effect to be used with the Orinj application
 effect framework.  The framework allows Orinj to automatically find and use
 digital signal processing effects (equalizers, reverbs, compressors, etc.).

 You can redistribute this file, modify it, adapt it, translate it, and/or distribute 
 the modified version, whether free of charge or not.  This file is distributed as is, without 
 warranty of any kind.
 
 @author      Anton Kamenov
 @version     3.0
*/

public interface EffectInterface
{
	/**
	 * Apply the effect taking a "dry" buffer and return a "wet" buffer
	 * @return			True, if there were no errors; false otherwise
	 * @param buffer  	The incoming sound data buffer to which the effect should be applied
	 *                  and the resulting outgoing sound data buffer with the effect applied
	 *
	 *                  Consider a delay effect.  The end result of a delay is a signal
	 *                  that contains both the "dry", original, not delayed signal and the
	 *                  "wet", delayed, and decayed signal.
	 *
	 *                  This is an array of integers with values between Integer.MIN_VALUE
	 *                  and Integer.MAX_VALUE, such as for 32-bit integer PCM sound data.
	 *                  This is so for any bit resolution that may be used in the rest of the
	 *                  application.  This applies to both the incoming data and the outgoing
     *                  data.
	 *
	 * @param controlbuffer	The control buffer should be used by effects that allow
	 * 					side chaining, such as a side chained compressor.  A side chained
	 * 					compressor changes the dynamics of one track based on the dynamics of
	 * 					a second track.  The audio data contained in the second track would
	 * 					be sent to the effect in this parameter.  An effect that uses this
	 * 					parameter should also return true in allowsSideChaining.  If the effect
	 * 					does not allow side chaining, this parameter may be null and should
	 * 					not be used.  The Orinj application controls which track is to be used
	 * 					as a control track and selecting a control track should not be implemented
	 * 					by the effect's graphical user interface.
     * @param channels  The number of audio channels in buffer.
     * @param channel   The channel to which this effect applies.  See EffectChannels
	 * @param samplingRate The sampling rate of the audio in buffer.
     * @param time      The start time in seconds of the audio buffers (e.g., buffer) in the
     *                  session or wave that is being played.  For example, if playback starts at
     *                  10 seconds into the multitrack session, time will be equal to 10 in the
     *                  first call to this function.
     * @param drymix    The dry mix of the effect.  This is a number between 0 and 1.  It is the
     *                  gain applied to the original signal before that original signal is placed
     *                  in buffer.  Effects that do not use dry or wet mix should not use this
     *                  argument.
     *
     *                  Not all effect have a dry and wet mix.  Consider a simple delay that
     *                  produces one delayed and decayed repetition of the original signal.
     *                  Implementing dry and wet mix in this effect is simple.  The dry mix gain
     *                  is applied to the original repetition and the wet mix gain is applied
     *                  to the delayed and decayed repetition.  Typically, the dry mix is only
     *                  applied to the original signal after the delayed and decayed repetition
     *                  is computed, so that the dry mix does not impact the wet portion of
     *                  the resulting signal.  Similarly, the wet mix should not impact the
     *                  dry, original portion of the signal.
     *
     *                  A compressor, on the other hand, will not typically have dry or wet mix.
     *                  In a compressor, the arguments drymix, wetmix, dryMixEnvelope, and
     *                  wetMixEnvelope can be ignored.
     * @param wetmix    The wet mix of the effect.  This is a number between 0 and 1.  It is the
     *                  gain applied to the processed signal after that processed signal is
     *                  computed from the original signal.  Effects that do not use dry or wet
     *                  mix should not use this argument.
     * @param dryMixEnvelope    A gain envelope for the dry mix of the effect.  This envelope
     *                  is an additional dry mix gain to be applied to the original signal
     *                  before this original signal is placed in buffer and after this original
     *                  signal is used to compute the wet signal.  Effects that do not have dry
     *                  and wet mix should not use this argument.
     *
     *                  The difference between this argument and drymix is that drymix is constant
     *                  throughout the audio data in buffer and the dry mix envelope can change.
     *                  A call to the member function of this argument Envelope::getValueAt
     *                  will return the gian to be applied at a specific time.  The gain is
     *                  between 0 and 1 and should be be used in the same way as drymix.
     * @param wetMixEnvelope    A gain envelope for the wet mix of the effect.  This envelope
     *                  is an additional wet mix gain to be applied to the processed signal
     *                  after this processed signal is computed from the original signal.
     *                  Effects that do not have dry and wet mix should not use this argument.
     *
     *                  The difference between this argument and wetmix is that wetmix is constant
     *                  throughout the audio data in buffer and the dry mix envelope can change.
     *                  A call to the member function of this argument Envelope::getValueAt
     *                  will return the gian to be applied at a specific time.  The gain is
     *                  between 0 and 1 and should be be used in the same way as wetmix.
     * @param timeStart The beginning of when the effect should be applied, in seconds, from the
     * 					beginning of the session or wave.  Effects should be expect that this 
     * 					function will not be called if the sound buffer is outside of the interval
     * 					between timeStart and timeEnd
     * @param timeEnd	The end of when the effect should be applied, in seconds, from the beginning
     * 					of the session or wave.  Effects should be expect that this 
     * 					function will not be called if the sound buffer is outside of the interval
     * 					between timeStart and timeEnd
     * @return true, if no errors; false, if errors
	 * @since 3.0
	 */
    public abstract boolean apply(int [] buffer, int [] controlbuffer, int channels, int channel, float samplingRate, 
    		double time, float drymix, float wetmix, EnvelopeInterface dryMixEnvelope, EnvelopeInterface wetMixEnvelope,
    		double timeStart, double timeEnd);
    
    /**
     * Check whether the effect allows dry / wet mix. See the description of the apply
     * function
     * @return  true, if the effect allows dry / wet mix; false otherwise
	 * @since 3.0
     */
    public abstract boolean allowsDryWetMix();

    /**
     * Check whether the effect allows side chaining and needs a control track.
     * A side chained compressor, for example, adjusts one track depending on the information
     * of another track (the control track).  The intent is for the application to provide buffers
     * with sound data to the effect for both tracks - the one to be changed and the one that
     * controls.  If the effect does not allow side chaining, the control buffer in the apply
     * function should be ignored (and may be null)
     * @return  true, if the effect allows side chaining; false otherwise
	 * @since 3.0
     */
    public abstract boolean allowsSideChaining();

    /**
     * Perform any preparations for playback.  The Orinj parametric equalizer,
     * for example, will compute all frequency filters dynamically here before
     * playback, so that it does not have to waste time computing those during
     * playback (until the user changes parameters, that is).
     * @return true if there were no errors; false otherwise
	 * @since 3.0
     */
    public abstract boolean startPlay();
    
    /**
     * Perform any cleanup after playback.  The Orinj multitap delay, for example,
     * will remove any of the arrays that are created to show where the impulse
     * delays are.
	 * @since 3.0
     */
    public abstract void stopPlay();

    /**
     * Return true or false depending on whether there is residual playback
     * data in the effect.  Orinj will ask whether the effect has residual data,
     * as some effects may continue playback after the original audio data is 
     * finished (i.e., the repetitions of an echo may sound even after the original
     * signal is done).
     * @return true, if there is residual data; false, otherwise
     */
    public abstract boolean hasData();
    
    /**
     * Read the effect from the serialization stream
     * @param stream The object input stream for serialization
     * @return  true, if there were no errors, false otherwise
	 * @since 3.0
     */
    public abstract boolean readObject(ReadInterface stream);
    
    /**
     * Write the effect to the serialization stream
     * @param stream The object output stream for serialization
     * @return  true, if there were no errors, false otherwise
     * @since 3.0
     */
    public abstract boolean writeObject(WriteInterface stream);
    
    /**
     * Makes this effect object equal to another effect of the same class.
     * @param effect  Another effect of the same class
     * @since 3.0
     */
    public abstract void setEqual(EffectInterface effect);
	
    /**
     * Set the language of the effect interface to be able to appropriate choose the strings
     * that should be shown for labels, tooltips, etc.  It is up to the author of the code
     * for the effect interfacer to decide how languages should be handled.  If not implementing
     * languages, leave this function empty.
     * @param languageCode Code for the language based on the three letter ISO 639-2
     */
    public abstract void setLanguage(String languageCode);
}
