package com.recordingblogs.oreffect;

import javax.swing.event.EventListenerList;

/**
 This class should be extended by any effect that wants to implement undo.  An effect 
 that extends this class would be able to add and remove UndoListener objects and 
 fire UndoEvent events.  The implementation of this class is standard.  Plug-in 
 effects in Orinj will be wrapped by a class that implements UndoListener and
 will catch UndoEvents to ensure that undo information is properly stored.  
 Extending this class is NOT sufficient to fully implement undo.  Effects must also 
 implement readObject(ReadInterface) and writeObject(ReadInterface) of the 
 EffectInterface class, which actually stores and reads undo information.  Effect GUI 
 panels must implement the updateData function of the EffectPanelInterface to ensure 
 that after undo the GUI of the effect is properly set (all effect dialogs are non-modal).

 You can redistribute this file, modify it, adapt it, translate it, and/or distribute 
 the modified version, whether free of charge or not.  This file is distributed as is, without 
 warranty of any kind.
  
 @author      Anton Kamenov
 @version     3.0
*/

public class Undo 
{
	/**
	 * An array of UndoListeners.  UndoEvent events will be fired at those listeners
	 * @since 3.0
	 */
	private EventListenerList m_listeners;

	/**
	 * The default and only constructor.  Create a new listener list
	 * @since 3.0
	 */
	public Undo()
	{
		m_listeners = new EventListenerList();
	}

	/**
	 * Add a new UndoListener to the array of listeners
	 * @param listener  The new UndoListener
	 * @since 3.0
	 */
	public void addUndoListener(UndoListener listener) 
	{
		m_listeners.add(UndoListener.class, listener);
	}

	/**
	 * Remove an UndoListener from the array of listeners
	 * @param listener  The UndoListener to be removed
	 * @since 3.0
	 */
	public void removeUndoListener(UndoListener listener) 
	{
		m_listeners.remove(UndoListener.class, listener);
	}

	/**
	 * Fire an UndoEvent event at each of the listeners in the array of UndoListener 
	 * objects.  Each listener will ensure that undo information is property stored
	 * @param event  The UndoEvent, containing the source object of the undo and a 
	 * 				 string description of the action that triggered the undo storage
	 * @since 3.0
	 */
	protected void fireUndoEvent(UndoEvent event) 
	{
		Object[] listeners = m_listeners.getListenerList();
		for (int i = 0; i < listeners.length; i += 2) 
		{
			if (listeners[i + 1] instanceof UndoListener)
				((UndoListener) listeners[i + 1]).undoStorageRequired(event);
		}
	}
}
